/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */
using System;
using System.Data;
using System.Data.SqlClient;
using CogArtTech.Music.Library.Lists;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library
{
    [Serializable]
    public class Catalog : BusinessBase<Catalog>
    {
        #region Properties

        private BandsList _Bands;
        private string _CountryCode = string.Empty;

        private Catalog()
        {
        }

        private Catalog(string countryCode)
        {
            _CountryCode = countryCode;
        }

        public string CountryCode
        {
            get
            {
                CanReadProperty(true);
                return _CountryCode;
            }
        }

        public BandsList Bands
        {
            get
            {
                CanReadProperty(true);
                return _Bands;
            }
        }

        #endregion

        #region Overrides

        protected override object GetIdValue()
        {
            return _CountryCode;
        }

        #endregion

        #region Factory Methods

        public static Catalog GetCatalog(string countryCode)
        {
            return DataPortal.Fetch<Catalog>(new Criteria(countryCode));
        }

        public static Catalog NewCatalog(string countryCode)
        {
            return DataPortal.Create<Catalog>(new Criteria(countryCode));
        }

        #endregion

        #region Data Access

        public override Catalog Save()
        {
            foreach (Band band in Bands)
            {
                band.Update();
            }
            return base.Save();
        }

        [RunLocal()]
        protected void DataPortal_Create(Criteria criteria)
        {
            _CountryCode = criteria.CountryCode;
            _Bands = BandsList.GetList(_CountryCode);
            ValidationRules.CheckRules();
        }

        [RunLocal()]
        private void DataPortal_Fetch(Criteria criteria)
        {
            this._CountryCode = criteria.CountryCode;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("GetBandsByCountry", cn))
                {
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("CountryCode", CountryCode);

                    using (SafeDataReader dr = new SafeDataReader(cmd.ExecuteReader()))
                    {
                        this._Bands = BandsList.GetList(dr);
                    }
                }
            } // using cn
            MarkOld();
            MarkClean();
        }

        #endregion

        #region Criteria

        protected class Criteria
        {
            private string _CountryCode;

            public string CountryCode
            {
                get { return _CountryCode; }
                set { _CountryCode = value; }
            }

            public Criteria()
            {
            }

            public Criteria(string countryCode)
            {
                this._CountryCode = countryCode;
            }
        }

        #endregion
    }
}