using System;
using System.Data;
using System.Data.SqlClient;
using CogArtTech.Music.Library.Lists;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library
{
    [Serializable]
    public class Instrument : BusinessBase<Instrument>
    {
        private int _Id = 0;
        private string _Description = string.Empty;

        private Instrument()
        {
            MarkAsChild();
        }

        internal static Instrument GetInstrument(SafeDataReader dr)
        {
            return new Instrument(dr);
        }

        private Instrument(SafeDataReader dr)
        {
            MarkAsChild();
            Fetch(dr);
        }

        internal static Instrument NewInstrument()
        {
            Instrument instrument = new Instrument();
            instrument.MarkNew();
            return instrument;
        }

        protected override object GetIdValue()
        {
            return _Id;
        }

        public int Id
        {
            get { return _Id; }
            set { _Id = value; }
        }

        public string Description
        {
            get { return _Description; }
            set
            {
                if (_Description != value)
                {
                    _Description = value;
                    PropertyHasChanged();
                }
                ;
            }
        }

        internal void DeleteSelf(Musician musician)
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;

            // if we're new then don't update the database
            if (this.IsNew) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();

                MarkNew();
            }
        }

        internal void Assign(Musician musician)
        {
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("usp_InsertInstrumentByMusician", cn))
                {
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@instrument_id", this.Id);
                    cmd.Parameters.AddWithValue("@musician_id", musician.Id);
                    SqlParameter param = new SqlParameter("instrument_musician_id", SqlDbType.Int);
                    param.Direction = ParameterDirection.Output;
                    cmd.Parameters.Add(param);
                    //cmd.ExecuteNonQuery();
                }
            }
        }

        internal void Update()
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("usp_UpdateInstrument", cn))
                {
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@instrument_id", this.Id);
                    cmd.Parameters.AddWithValue("@description", this.Description);
                    cmd.ExecuteNonQuery();
                }
            }
            MarkClean();
            MarkOld();
            InstrumentsList.InvalidateCache();
        }

        internal void Insert()
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("usp_InsertInstrument", cn))
                {
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("@description", this.Description);
                    SqlParameter param =
                        new SqlParameter("@instrument_id", SqlDbType.Int);
                    param.Direction = ParameterDirection.Output;
                    cmd.Parameters.Add(param);
                    cmd.ExecuteNonQuery();
                    this.Id = Convert.ToInt32(cmd.Parameters["@instrument_id"].Value);
                }
            }
            MarkClean();
            MarkOld();
            InstrumentsList.InvalidateCache();
        }

        private void Fetch(SafeDataReader dr)
        {
            _Id = dr.GetInt32("instrument_id");
            _Description = dr.GetString("Description");
            MarkOld();
            MarkClean();
        }
    }
}