/*
 * Created by: 
 * Created: Sunday, March 25, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class BandSongsList : BusinessListBase<BandSongsList, Song>
    {
        private static Band _Band = null;

        #region Business Methods

        public static BandSongsList GetList(Band band)
        {
            _Band = band;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cm = cn.CreateCommand())
                {
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "usp_SelectSongByBand";
                    cm.Parameters.AddWithValue("@band_id", _Band.Id);

                    using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
                    {
                        return GetList(dr);
                    }
                }
            }
        }

        public Song GetItem(int songId)
        {
            foreach (Song song in this)
                if (song.Id == songId)
                    return song;
            return null;
        }

        public void Unassign(Song song)
        {
            Unassign(song.Id);
        }

        public void Unassign(int songId)
        {
            Remove(songId);
        }

        public void Assign(Song song)
        {
            Assign(song.Id);
        }

        public void Assign(int songId)
        {
            if (!Contains(songId))
            {
                foreach (Song song in SongsList.GetList())
                {
                    if (song.Id == songId)
                    {
                        Song newSong = Song.NewSong(song.Id, song.Name, song.RuntimeSeconds);
                        this.Add(newSong);
                    }
                }
            }
            else
                throw new InvalidOperationException(
                    "Song already assigned to band");
        }

        private new void Remove(Song song)
        {
            base.Remove(song);
        }

        private void Remove(int songId)
        {
            foreach (Song song in this)
            {
                if (song.Id == songId)
                {
                    Remove(song);
                    break;
                }
            }
        }

        public bool Contains(int songId)
        {
            foreach (Song song in this)
                if (song.Id == songId)
                    return true;
            return false;
        }

        public bool ContainsDeleted(int songId)
        {
            foreach (Song song in DeletedList)
                if (song.Id == songId)
                    return true;
            return false;
        }

        private new void Add(Song song)
        {
            base.Add(song);
        }

        #endregion

        #region Factory Methods

        public Band Band
        {
            get { return _Band; }
        }

        internal static BandSongsList NewList()
        {
            return new BandSongsList();
        }

        internal static BandSongsList GetList(SafeDataReader dr)
        {
            return new BandSongsList(dr);
        }

        private BandSongsList()
        {
            MarkAsChild();
        }

        private BandSongsList(SafeDataReader dr)
        {
            MarkAsChild();
            Fetch(dr);
        }

        #endregion

        #region Data Access

        // called to load data from the database

        private void Fetch(SafeDataReader dr)
        {
            this.RaiseListChangedEvents = false;
            this.Clear();
            while (dr.Read())
                this.Add(Song.GetSong(dr));
            this.RaiseListChangedEvents = true;
        }

        private void RemoveSongFromBand(Band band, int songId)
        {
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                SqlCommand cmd = new SqlCommand();
                cmd.Connection = cn;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.CommandText = "usp_DeleteSongByBand";
                cmd.Parameters.AddWithValue("@song_id", songId);
                cmd.Parameters.AddWithValue("@band_id", band.Id);

                cmd.ExecuteNonQuery();
            }
        }

        internal void Update(Band band)
        {
            this.RaiseListChangedEvents = false;
            // update (thus deleting) any deleted child objects
            foreach (Song obj in DeletedList)
            {
                RemoveSongFromBand(band, obj.Id);
            }

            // now that they are deleted, remove them from memory too
            DeletedList.Clear();
            // add/update any current child objects
            foreach (Song obj in this)
            {
                if (obj.IsNew)
                {
                    // Now add the instrument to the list
                    using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
                    {
                        cn.Open();
                        using (SqlCommand cmd = new SqlCommand("usp_InsertSongByBand", cn))
                        {
                            cmd.Connection = cn;
                            cmd.CommandType = CommandType.StoredProcedure;
                            cmd.Parameters.AddWithValue("@song_id", obj.Id);
                            cmd.Parameters.AddWithValue("@band_id", band.Id);

                            SqlParameter param =
                                new SqlParameter("@song_band_id", SqlDbType.Int);
                            param.Direction = ParameterDirection.Output;
                            cmd.Parameters.Add(param);
                            cmd.ExecuteNonQuery();
                        }
                    }
                }
            }
            this.RaiseListChangedEvents = true;
        }

        #endregion
    }
}