/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class BandsList : BusinessListBase<BandsList, Band>
    {

        #region Constructors

        private BandsList()
        {
            MarkAsChild();
        }

        private BandsList(SafeDataReader dr)
            : this()
        {
            Fetch(dr);
        }

        #endregion
        
        #region Business Methods

        public Band GetItem(int bandId)
        {
            foreach (Band band in this)
                if (band.Id == bandId)
                    return band;
            return null;
        }

        public void Assign(int bandId)
        {
            if (!Contains(bandId))
            {
                Band band = Band.NewBand();
                this.Add(band);
            }
            else
                throw new InvalidOperationException(
                    "Band already assigned to catalog");
        }

        public void Remove(int bandId)
        {
            foreach (Band band in this)
            {
                if (band.Id == bandId)
                {
                    Remove(band);
                    break;
                }
            }
        }

        public bool Contains(int bandId)
        {
            foreach (Band band in this)
                if (band.Id == bandId)
                    return true;
            return false;
        }

        public bool ContainsDeleted(int bandId)
        {
            foreach (Band band in DeletedList)
                if (band.Id == bandId)
                    return true;
            return false;
        }

        #endregion

        #region Factory Methods

        internal static BandsList GetList(SafeDataReader dr)
        {
            return new BandsList(dr);
        }

        #endregion

        #region Data Access

        // called to load data from the database
        private void Fetch(SafeDataReader dr)
        {
            this.RaiseListChangedEvents = false;
            while (dr.Read())
                this.Add(Band.GetBand(dr));
            this.RaiseListChangedEvents = true;
        }

        internal void Update()
        {
            this.RaiseListChangedEvents = false;
            // update (thus deleting) any deleted child objects
            foreach (Band obj in DeletedList)
                obj.DeleteSelf();
            // now that they are deleted, remove them from memory too
            DeletedList.Clear();

            // add/update any current child objects
            foreach (Band obj in this)
            {
                if (obj.IsNew)
                    obj.Insert();
                else
                    obj.Update();
            }
            this.RaiseListChangedEvents = true;
        }

        protected override void DataPortal_Update()
        {
            Update();
        }

        /// <summary>
        /// Returns a list of all known bands
        /// </summary>
        /// <returns><see cref="BandsList"/> of all known bands</returns>
        public static BandsList GeList()
        {
            BandsList list;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cm = cn.CreateCommand())
                {
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "GetBands";


                    using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
                    {
                        dr.Read();
                        list = new BandsList(dr);
                    }
                }
            }
            return list;
        }

        public static BandsList GetList(string countryCode)
        {
            BandsList list;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cm = cn.CreateCommand())
                {
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "GetBandsByCountry";
                    cm.Parameters.AddWithValue("@CountryCode", countryCode);
                    using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
                    {
                        list = new BandsList(dr);
                    }
                }
            }
            return list;
        }

        public static Band NewBand()
        {
            return Band.NewBand();
        }

        #endregion
    }
}