/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class InstrumentsList : BusinessListBase<InstrumentsList, Instrument>
    {
        private static InstrumentsList _List;

        #region Business Methods

        internal static void InvalidateCache()
        {
            _List = null;
        }

        #endregion

        #region Factory Methods

        private InstrumentsList()
        {
            Fetch();
        }

        public static InstrumentsList GetList()
        {
            if (_List == null)
            {
                _List = new InstrumentsList();
            }
            return _List;
        }

        #endregion

        #region Data Access

        /// <summary>
        /// DataPortal_Update is called by the framwork when the root
        /// class new items added to it, or when an existing item is updated.
        /// </summary>
        [RunLocal]
        protected override void DataPortal_Update()
        {
            Update();
        }

        /// <summary>
        /// DataPortal_Fetch is fundamental to the framework. It is called
        /// by CSLA internals when loading data. Data loads are instigated from
        /// the root class. In this case the list itself is the root class.
        /// </summary>
        /// <param name="criteria">The criteria.</param>
        [RunLocal]
        protected override void DataPortal_Fetch(object criteria)
        {
            //base.DataPortal_Fetch(criteria);
            Fetch();
        }

        private void Fetch()
        {
            this.RaiseListChangedEvents = false;
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cm = cn.CreateCommand())
                {
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "GetInstruments";

                    using (SafeDataReader dr =
                        new SafeDataReader(cm.ExecuteReader()))
                    {
                        //IsReadOnly = false;
                        while (dr.Read())
                        {
                            this.Add(Instrument.GetInstrument(dr));
                        }
                        //IsReadOnly = true;
                    }
                }
            }
            this.RaiseListChangedEvents = true;
        }

        #endregion

        internal void Update()
        {
            this.RaiseListChangedEvents = false;
            // add/update any current child objects
            foreach (Instrument obj in this)
            {
                if (obj.IsNew)
                {
                    obj.Insert();
                }

                if (obj.IsDirty)
                {
                    obj.Update();
                }
            }
            this.RaiseListChangedEvents = true;
        }

        public static Instrument NewInstrument()
        {
            return Instrument.NewInstrument();
        }
    }
}