/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class SongsList : BusinessListBase<SongsList, Song>
    {
        

        private static SongsList _List = null;

        #region Business Methods

        public Song GetItem(int songId)
        {
            foreach (Song song in this)
                if (song.Id == songId)
                    return song;
            return null;
        }

        public bool Contains(int songId)
        {
            foreach (Song song in this)
                if (song.Id == songId)
                    return true;
            return false;
        }

        #endregion

        #region Factory Methods

        public static SongsList GetList()
        {
            if (_List == null)
            {
                _List = new SongsList();
            }
            return _List;
        }

        private SongsList()
        {
            Fetch();
        }

        #endregion

        #region Data Access

        /// <summary>
        /// This method is a part of the CSLA framework. You fill in 
        /// update behavior here.
        /// </summary>
        protected override void DataPortal_Update()
        {
            Update();
        }
        
        // called to load data from the database
        private void Fetch()
        {
            this.RaiseListChangedEvents = false;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cm = cn.CreateCommand())
                {
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "GetSongs";

                    using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
                    {
                        while (dr.Read())
                            this.Add(Song.GetSong(dr));
                        this.RaiseListChangedEvents = true;
                    }
                }
            }
        }

        internal void Update()
        {
            this.RaiseListChangedEvents = false;
            // update (thus deleting) any deleted child objects
            foreach (Song obj in DeletedList)
                obj.DeleteSelf(); // the SongsByBand deletes will cascade

            // now that they are deleted, remove them from memory too
            DeletedList.Clear();

            // add/update any current child objects
            foreach (Song obj in this)
            {
                if (obj.IsNew)
                {
                    obj.Insert();
                }

                if (obj.IsDirty)
                {
                    obj.Update();
                }
            }
            this.RaiseListChangedEvents = true;
        }

        #endregion

        #region New Object Method

        public static Song NewSong(string name, int runtime)
        {
            Song song = Song.NewSong(0, name, runtime);

            return song;
        }

        #endregion
    }
}