/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using CogArtTech.Music.Library.Lists;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library
{
    [Serializable]
    public class Musician : BusinessBase<Musician>
    {
        private string _FirstName = string.Empty;
        private string _LastName = string.Empty;
        private SmartDate _DateOfBirth = new SmartDate();
        private int _Id = 0;
        private MusicianInstrumentsList _Instruments = null;

        private Musician()
        {
        }

        private Musician(SafeDataReader dr)
        {
            MarkAsChild();
            Fetch(dr);
        }

        internal static Musician GetMusician(SafeDataReader dr)
        {
            return new Musician(dr);
        }

        internal static Musician NewMusician()
        {
            Musician musician = new Musician();
            musician.MarkNew();
            musician.MarkAsChild();
            return musician;
        }

        public MusicianInstrumentsList Instruments
        {
            get
            {
                if (_Instruments == null)
                {
                    _Instruments = MusicianInstrumentsList.NewMusicianInstrumentsList();
                }
                return _Instruments;
            }
        }

        protected override object GetIdValue()
        {
            return _Id;
        }

        public string FirstName
        {
            get { return _FirstName; }
            set
            {
                if (_FirstName != value)
                {
                    _FirstName = value;
                    PropertyHasChanged();
                }
            }
        }

        public string LastName
        {
            get { return _LastName; }
            set
            {
                if (_LastName != value)
                {
                    _LastName = value;
                    PropertyHasChanged();
                }
            }
        }

        public string FullName
        {
            get { return string.Format("{0}, {1}", _LastName, _FirstName); }
        }

        public SmartDate DateOfBirth
        {
            get { return _DateOfBirth; }
            set
            {
                if (_DateOfBirth != value)
                {
                    _DateOfBirth = value;
                    PropertyHasChanged();
                }
            }
        }

        public int Id
        {
            get { return _Id; }
            set
            {
                if (_Id != value)
                {
                    _Id = value;
                    PropertyHasChanged();
                }
            }
        }

        internal void DeleteSelf(Band band)
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;

            // if we're new then don't update the database
            if (this.IsNew) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                MarkNew();
                throw new NotImplementedException();
            }
        }

        internal void DeleteSelf()
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;

            // if we're new then don't update the database
            if (this.IsNew) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                MarkNew();
                throw new NotImplementedException();
            }
        }

        internal void Insert()
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand())
                {
                    cmd.Connection = cn;
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = "usp_InsertMusician";
                    cmd.Parameters.AddWithValue("@first_name", this.FirstName);
                    cmd.Parameters.AddWithValue("@last_name", this.LastName);
                    cmd.Parameters.AddWithValue("@date_of_birth", this.DateOfBirth.Date);
                    SqlParameter param =
                        new SqlParameter("@musician_id", SqlDbType.Int);
                    param.Direction = ParameterDirection.Output;
                    cmd.Parameters.Add(param);
                    cmd.ExecuteNonQuery();
                    this.Id = Convert.ToInt32(cmd.Parameters["@musician_id"].Value);
                    MarkOld();
                    MarkClean();
                }
            }
        }

        internal void Update()
        {
            // if we're not dirty then don't update the database
            //if (!this.IsDirty) return;
            if (base.IsDirty)
            {
                using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
                {
                    cn.Open();
                    SqlCommand cm = new SqlCommand();
                    cm.CommandType = CommandType.StoredProcedure;
                    cm.CommandText = "usp_UpdateMusician";
                    cm.Parameters.AddWithValue("@musician)id", this.Id);
                    cm.Parameters.AddWithValue("@first_name", this.FirstName);
                    cm.Parameters.AddWithValue("@last_name", this.LastName);
                    cm.Parameters.AddWithValue("@date_of_birth", this.DateOfBirth);
                    cm.ExecuteNonQuery();
                    this.Id = Convert.ToInt32(cm.Parameters["@musician_id"]);
                    MarkOld();
                    MarkClean();
                }
            }
            //update the instruments
            if (this.Instruments.IsDirty)
            {
                this.Instruments.Update(this);
            }
        }

        internal void Assign(Band band)
        {
            // if we're not dirty then don't update the database
            if (!this.IsDirty) return;

            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                SqlCommand cmd = new SqlCommand();
                cmd.Connection = cn;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.CommandText = "usp_InsertMusicianByBand";
                cmd.Parameters.AddWithValue("@musician_id", this.Id);
                cmd.Parameters.AddWithValue("@band_id", band.Id);
                SqlParameter param =
                    new SqlParameter("@musician_band_id", SqlDbType.Int);
                param.Direction = ParameterDirection.Output;
                cmd.Parameters.Add(param);
                cmd.ExecuteNonQuery();
                MarkOld();
            }
            //update the instruments

            if (this.Instruments.IsDirty)
            {
                this.Instruments.Update(this);
            }
        }

        private void Fetch(SafeDataReader dr)
        {
            _Id = dr.GetInt32("musician_id");
            _FirstName = dr.GetString("first_name");
            _LastName = dr.GetString("last_name");
            if (dr["date_of_birth"] != DBNull.Value)
            {
                _DateOfBirth = new SmartDate(Convert.ToDateTime(dr["date_of_birth"]));
            }
            else
            {
                _DateOfBirth = new SmartDate();
            }
            //get instruments

            BindInstruments();
            MarkOld();
            MarkClean();
        }

        private void BindInstruments()
        {
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                using (SqlCommand cmd = new SqlCommand("usp_SelectInstrumentByMusician", cn))
                {
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.Parameters.AddWithValue("musician_id", this.Id);

                    using (SafeDataReader dr = new SafeDataReader(cmd.ExecuteReader()))
                    {
                        _Instruments = MusicianInstrumentsList.GetList(dr, this);
                    }
                }
            } // using cn
        }

        public override bool IsDirty
        {
            get { return base.IsDirty || _Instruments.IsDirty; }
        }
    }
}