using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library
{
   [Serializable]
   public class Song : BusinessBase<Song>
   {
      private int _Id = 0;
      private string _Name = string.Empty;
      private int _RuntimeSeconds = 0;

      private Song()
      {

          MarkAsChild();
      }

      private Song(SafeDataReader dr)
      {
         MarkAsChild();
         Fetch(dr);
      }

      internal static Song GetSong(SafeDataReader dr)
      {
         return new Song(dr);
      }

      internal static Song NewSong(int id, string name, int runtTimeSeconds)
      {
         Song song = new Song();
         song.Id = id;
         song.Name = name;
         song.RuntimeSeconds = runtTimeSeconds;
         song.MarkNew();
         return song;
      }

      public int Id
      {
         get { return _Id; }
         set { _Id = value; }
      }

      public string Name
      {
         get { return _Name; }
         set
         {
             if(_Name!=value)
             {
                _Name = value; 
                 PropertyHasChanged();
             }
             
         }
      }

      public int RuntimeSeconds
      {
         get { return _RuntimeSeconds; }
         set
         {
             if (_RuntimeSeconds!=value)
             {
                 _RuntimeSeconds = value;
                 PropertyHasChanged(); 
             }
         }
      }

      protected override object GetIdValue()
      {
         return _Id;
      }

      internal void DeleteSelf()
      {
         // if we're not dirty then don't update the database
         if (!this.IsDirty) return;

         // if we're new then don't update the database
         if (this.IsNew) return;

         using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
         {
            cn.Open();

            MarkNew();
         }
      }

      internal void Insert()
      {
         // if we're not dirty then don't update the database
         if (!this.IsDirty) return;

         using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
         {
            cn.Open();
            using (SqlCommand cmd = new SqlCommand("usp_InsertSong", cn))
            {
               cmd.CommandType = CommandType.StoredProcedure;
               cmd.Parameters.AddWithValue("@run_time_seconds", this.RuntimeSeconds);
               cmd.Parameters.AddWithValue("@name", this.Name);
               SqlParameter param =
                 new SqlParameter("@song_id", SqlDbType.Int);
               param.Direction = ParameterDirection.Output;
               cmd.Parameters.Add(param);
               cmd.ExecuteNonQuery();
               if(cmd.Parameters["@song_id"].Value==DBNull.Value)
               {
                   throw new ArgumentException("No Id returned on object save.");
               }
               this.Id = Convert.ToInt32(cmd.Parameters["@song_id"].Value);
            }
         }
         MarkOld();
      }

      internal void Update()
      {
         // if we're not dirty then don't update the database
         if (!this.IsDirty) return;

         using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
         {
            cn.Open();
            using (SqlCommand cmd = new SqlCommand())
            {
               cmd.Connection = cn;
               cmd.CommandType = CommandType.StoredProcedure;
               cmd.CommandText = "usp_UpdateSong";
               cmd.Parameters.AddWithValue("@run_time_seconds", this.RuntimeSeconds);
               cmd.Parameters.AddWithValue("@name", this.Name);
               cmd.Parameters.AddWithValue("@song_id", this.Id);
               cmd.ExecuteNonQuery();
            }
         }
         MarkClean();
         MarkOld();
      }

      private void Fetch(SafeDataReader dr)
      {
         _Id = dr.GetInt32("song_id");
         _Name = dr.GetString("name");
         _RuntimeSeconds = dr.GetInt32("run_time_seconds");
         MarkOld();
      }
       
      public void SaveSong()
       {
           if(this.Id>0)
           {
               Update();
           }else
           {
               Insert();
           }
       }
   }
}