using System;
using System.Collections.ObjectModel;
using System.Data;
using System.Web.UI.WebControls;
using CogArtTech.Music.Library;
using CogArtTech.Music.Library.Lists;
using Csla.Data;

namespace CogArtTech.Music.WebApp
{
    public partial class _Default : MusicPageBase
    {
        Catalog _Catalog = null;
        
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                SetBandsToDropDown(rlistCountries.SelectedValue);
                SetSongList();
                SetMusiciansList();
                SetInstruments();
            }
        }

        private void SetMusicianByBand(int bandId)
        {
        }

        private void SetMusiciansList()
        {
            //Get the a musicians BusinessBaseList
            MusiciansList musicians = MusiciansList.GetList();

            //Bind the DataSet to the musician's 
            //DropDownList
            lstMusicians.DataSource = musicians;
            lstMusicians.DataTextField = "FullName";
            lstMusicians.DataValueField = "Id";
            lstMusicians.DataBind();
        }

        private void SetSongList()
        {
            ObjectAdapter adapter = new ObjectAdapter();
            DataSet ds = new DataSet();
            SongsList songs = SongsList.GetList();
            adapter.Fill(ds, songs);
            lstSongs.DataSource = songs;
            lstSongs.DataTextField = "Name";
            lstSongs.DataValueField = "Id";
            lstSongs.DataBind();
        }

        private void SetBandsToDropDown(string countryCode)
        {
            this.CurrentBands = BandsList.GetList(countryCode);
            ddBands.DataSource = this.CurrentBands;
            ddBands.DataTextField = "Name";
            ddBands.DataValueField = "Id";
            ddBands.DataBind();
            ddBands.Items.Insert(0, new ListItem("Select", "SELECT"));
        }

        /// <summary>
        /// Sets the data to drop down.
        /// </summary>
        /// <param name="listControl">The ListControl.</param>
        /// <param name="dataTextField">The data text field.</param>
        /// <param name="dataValueField">The data value field.</param>
        /// <param name="adapterSource">The adapter source.</param>
        /// Created: 4/22/2007 8:42 PM
        /// Created By: Bob Reselman
        private void SetDataToList(ListControl listControl, string dataTextField,
                                   string dataValueField, object adapterSource)
        {
            DataSet ds;
            ds = new DataSet();
            ObjectAdapter adapter;
            adapter = new ObjectAdapter();
            adapter.Fill(ds, adapterSource);

            listControl.DataSource = adapterSource;
            listControl.DataTextField = dataTextField;
            listControl.DataValueField = dataValueField;
            listControl.DataBind();
        }


        protected void rlistCountries_SelectedIndexChanged(object sender, EventArgs e)
        {
            RadioButtonList list = (RadioButtonList) sender;
            SetBandsToDropDown(list.SelectedValue);
        }

        private void SetInstruments()
        {
            InstrumentsList instrumentList = InstrumentsList.GetList();
            lstInstruments.DataSource = instrumentList;
            lstInstruments.DataTextField = "Description";
            lstInstruments.DataValueField = "Id";
            lstInstruments.DataBind();
        }


        protected void btnSongAdd_Click(object sender, EventArgs e)
        {
            if (lstSongs.SelectedItem != null)
            {
                ListItem item = lstSongs.SelectedItem;
                ListItem buffer = new ListItem();
                buffer.Text = item.Text;
                buffer.Value = item.Value;
                lstBandSongs.Items.Add(buffer);
            }
        }

        protected void btnMemberAdd_Click(object sender, EventArgs e)
        {
            if (lstMusicians.SelectedItem != null)
            {
                ListItem item = lstMusicians.SelectedItem;
                ListItem buffer = new ListItem();
                buffer.Text = item.Text;
                buffer.Value = item.Value;
                lstMembers.Items.Add(buffer);
            }
        }

        protected void btnMemberDelete_Click(object sender, EventArgs e)
        {
            if (lstMembers.SelectedItem != null)
            {
                lstMembers.Items.Remove(lstMembers.SelectedItem);
            }
        }

        protected void btnSongDelete_Click(object sender, EventArgs e)
        {
            if (lstBandSongs.SelectedItem != null)
            {
                lstBandSongs.Items.Remove(lstBandSongs.SelectedItem);
            }
        }

        protected void lbBandDetail_Click(object sender, EventArgs e)
        {
            Server.Transfer("BandDetail.aspx");
        }

        protected void lbMusicianAdd_Click(object sender, EventArgs e)
        {
            Server.Transfer("MusicianDetail.aspx");
        }

        protected void lbSongAdd_Click(object sender, EventArgs e)
        {
            Server.Transfer("SongDetail.aspx");
        }

        protected void lstMusicians_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentMusicianId = Convert.ToInt32(lstMusicians.SelectedItem.Value);
            lblMusician.Text = lstMusicians.SelectedItem.Text;
        }

        protected void lstMembers_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstMembers.SelectedItem != null)
            {
                CurrentMusicianId = Convert.ToInt32(lstMembers.SelectedItem.Value);
                lblMusician.Text = lstMembers.SelectedItem.Text;
                foreach (Musician musician in CurrentBand.Members)
                {
                    if (musician.Id == CurrentMusicianId)
                    {
                        SetDataToList(lstMusicianInstruments, "Description", "Id", musician.Instruments);
                    }
                }

                EnableInstruments(true);
            }
        }

        protected void btnInstrumentAdd_Click(object sender, EventArgs e)
        {
            if (lstInstruments.SelectedItem != null && CurrentMusicianId > 0)
            {
                ListItem item = lstInstruments.SelectedItem;
                ListItem buffer = new ListItem();
                buffer.Text = item.Text;
                buffer.Value = item.Value;
                lstMusicianInstruments.Items.Add(buffer);
            }
        }

        protected void btnInstrumentRemove_Click(object sender, EventArgs e)
        {
            if (lstMusicianInstruments.SelectedItem != null)
            {
                lstMusicianInstruments.Items.Remove(lstMusicianInstruments.SelectedItem);
            }
        }


        protected void ddBands_SelectedIndexChanged(object sender, EventArgs e)
        {
            ClearWriteLists();
            if (ddBands.SelectedValue != "SELECT")
            {
                foreach (Band band in CurrentBands)
                {
                    if (band.Id == Convert.ToInt32(ddBands.SelectedValue))
                    {
                        this.CurrentBand = band;
                        break;
                    }
                }

                //Enable the add and remove buttons for musician and song
                EnableMusiciansAndSongs(true);

                //Populate the members
                if (this.CurrentBand.Members != null)
                {
                    SetDataToList(lstMembers, "FullName", "Id", this.CurrentBand.Members);
                }


                //Populate the songs
                if (this.CurrentBand.Songs != null)
                {
                    SetDataToList(lstBandSongs, "Name", "Id", this.CurrentBand.Songs);
                }
            }
        }

        private void EnableMusiciansAndSongs(bool isEnabled)
        {
            btnSongAdd.Enabled = isEnabled;
            btnMemberAdd.Enabled = isEnabled;

            btnSongRemove.Enabled = isEnabled;
            btnMemberRemove.Enabled = isEnabled;
        }

        private void EnableInstruments(bool isEnabled)
        {
            btnInstrumentAdd.Enabled = isEnabled;
            btnInstrumentRemove.Enabled = isEnabled;
        }

        protected void btnUpdate_Click(object sender, EventArgs e)
        {
            _Catalog = Catalog.GetCatalog(rlistCountries.SelectedValue);

            BandsList bandsList = _Catalog.Bands;
            foreach (Band band in bandsList)
            {
                if (ddBands.SelectedIndex!=0 &&  band.Id == Int32.Parse(ddBands.SelectedValue))
               {
                   //this is the band to process
                   ProcessBand(band);
               } 
            }

            _Catalog.Save();
            _Catalog = Catalog.GetCatalog(rlistCountries.SelectedValue);
            
        }
        
                
      private void ClearWriteLists()
      {
          lstBandSongs.Items.Clear();
          lstMembers.Items.Clear();
          lstMusicianInstruments.Items.Clear();
      }
        
        private void ProcessBand(Band band)
        {
            
            MusiciansList musicians = MusiciansList.GetList();
            SongsList songs = SongsList.GetList();
            
            //Process Songs, add the ones that are new
            foreach(ListItem item in lstBandSongs.Items)
            {
                Song song = GetSongFromList(songs, int.Parse(item.Value));
                if(!band.Songs.Contains(song))
                {
                    band.Songs.Assign(song);
                }
            }
            //Process musicians
            foreach (ListItem item in lstMembers.Items)
            {
                Musician musician = GetMusicianFromList(musicians, int.Parse(item.Value));
                if (!band.Members.Contains(musician))
                {
                    band.Members.Assign(musician);
                }
            }
            
            Collection<int> musicianIdsToNuke = new Collection<int>();
            Collection<int> songIdsToNuke = new Collection<int>();
            
            //remove the ones that need to be removed
            foreach (Musician musician in band.Members)
            {
                if(!IsIdInList(lstMembers,musician.Id))
                {
                    if (!musicianIdsToNuke.Contains(musician.Id))
                    {
                       musicianIdsToNuke.Add(musician.Id); 
                    } 
                }
            }

            foreach (Song song in band.Songs)
            {
                if (!IsIdInList(lstBandSongs, song.Id))
                {
                    if (!songIdsToNuke.Contains(song.Id))
                    {
                        songIdsToNuke.Add(song.Id);
                    }
                }
            }
            
            if(musicianIdsToNuke.Count>0)
            {
                foreach(int id in musicianIdsToNuke)
                {
                    band.Members.Unassign(id);
                }
            }

            if (songIdsToNuke.Count > 0)
            {
                foreach (int id in songIdsToNuke)
                {
                    band.Songs.Unassign(id);
                }
            }
        }
        
        private bool IsIdInList(ListBox list, int id)
        {
            foreach (ListItem item in list.Items)
            {
                if(item.Value == id.ToString())
                {
                   return true; 
                }
            }
            return false;
        }

        #region List Retrieval Methods
        private Musician GetMusicianFromList(MusiciansList list, int musicianId)
        {
            foreach (Musician musician in list)
            {
                if (musician.Id == musicianId)
                {
                    return musician;
                }
            }
            return null;
        }

        private Song GetSongFromList(SongsList list, int songId)
        {
            foreach (Song song in list)
            {
                if (song.Id == songId)
                {
                    return song;
                }
            }
            return null;
        }

        private Instrument GetInstrumentFromList(InstrumentsList list, int instrumentId)
        {
            foreach (Instrument instrument in list)
            {
                if (instrument.Id == instrumentId)
                {
                    return instrument;
                }
            }
            return null;
        }
        
        #endregion
        private BandsList CurrentBands
        {
            get { return (BandsList) ViewState["CurrentBands"]; }
            set { ViewState["CurrentBands"] = value; }
        }

        private Band CurrentBand
        {
            get { return (Band) ViewState["CurrentBand"]; }
            set { ViewState["CurrentBand"] = value; }
        }

        protected void btnUpdateInstrumentMusician_Click(object sender, EventArgs e)
        {
            if(lstMembers.SelectedItem != null)
            {
                MusiciansList musicians = MusiciansList.GetList();
                InstrumentsList instruments = InstrumentsList.GetList();
                Musician musician = GetMusicianFromList(musicians, int.Parse(lstMembers.SelectedValue));
                bool canSave = false;
                foreach (ListItem instrumentItem in lstMusicianInstruments.Items)
                {
                    Instrument instrument = GetInstrumentFromList(instruments, int.Parse(instrumentItem.Value));

                    if (!musician.Instruments.Contains(instrument))
                    {
                        musician.Instruments.Assign(instrument);
                        canSave = true;
                    }
                }
                Collection<int> instrumentIdsToNuke = new Collection<int>();
                foreach (Instrument instrument in musician.Instruments)
                {
                    if (!IsIdInList(lstMusicianInstruments, instrument.Id))
                    {
                        if (!instrumentIdsToNuke.Contains(instrument.Id))
                        {
                            instrumentIdsToNuke.Add(instrument.Id);
                            canSave = true;
                        }
                    }
                }

                if (instrumentIdsToNuke.Count > 0)
                {
                    foreach (int id in instrumentIdsToNuke)
                    {
                        musician.Instruments.Unassign(id);
                    }
                }
                
                if(canSave)
                {
                    musicians.Save();
                }

            }

        }
    }
}